@@ -1,5 +1,6 @@
 from django import test
 from django.conf import settings
+from django.contrib.auth import REDIRECT_FIELD_NAME
 from django.core.urlresolvers import reverse
 
 from keystoneclient import exceptions as keystone_exceptions
@@ -39,10 +40,12 @@ class OpenStackAuthTests(test.TestCase):
         client.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                       password=user.password,
                       username=user.name,
+                      insecure=False,
                       tenant_id=None).AndReturn(self.keystone_client)
         self.keystone_client.tenants.list().AndReturn(tenants)
         client.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                       tenant_id=self.data.tenant_two.id,
+                      insecure=False,
                       token=sc.get_token()['id']) \
                 .AndReturn(self.keystone_client)
         self.keystone_client.tokens.authenticate(tenant_id=tenants[1].id,
@@ -75,6 +78,7 @@ class OpenStackAuthTests(test.TestCase):
         client.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                       password=user.password,
                       username=user.name,
+                      insecure=False,
                       tenant_id=None).AndReturn(self.keystone_client)
         self.keystone_client.tenants.list().AndReturn([])
 
@@ -105,6 +109,7 @@ class OpenStackAuthTests(test.TestCase):
         client.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                       password="invalid",
                       username=user.name,
+                      insecure=False,
                       tenant_id=None).AndRaise(exc)
 
         self.mox.ReplayAll()
@@ -133,6 +138,7 @@ class OpenStackAuthTests(test.TestCase):
         client.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                       password=user.password,
                       username=user.name,
+                      insecure=False,
                       tenant_id=None).AndRaise(exc)
 
         self.mox.ReplayAll()
@@ -151,7 +157,7 @@ class OpenStackAuthTests(test.TestCase):
                             ("An error occurred authenticating. Please try "
                              "again later."))
 
-    def test_switch(self):
+    def test_switch(self, next=None):
         tenant = self.data.tenant_two
         tenants = [self.data.tenant_one, self.data.tenant_two]
         user = self.data.user
@@ -169,6 +175,7 @@ class OpenStackAuthTests(test.TestCase):
         client.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                       password=user.password,
                       username=user.name,
+                      insecure=False,
                       tenant_id=None).AndReturn(self.keystone_client)
         self.keystone_client.tenants.list().AndReturn(tenants)
         self.keystone_client.tokens.authenticate(tenant_id=tenants[1].id,
@@ -178,10 +185,12 @@ class OpenStackAuthTests(test.TestCase):
 
         client.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                       tenant_id=self.data.tenant_two.id,
+                      insecure=False,
                       token=sc.get_token()['id']) \
                 .AndReturn(self.keystone_client)
 
-        client.Client(endpoint=sc.url_for()) \
+        client.Client(endpoint=sc.url_for(),
+                      insecure=False) \
                 .AndReturn(self.keystone_client)
 
         self.keystone_client.tokens.authenticate(tenant_id=tenant.id,
@@ -204,8 +213,19 @@ class OpenStackAuthTests(test.TestCase):
         sc.catalog['token']['id'] = self.data.tenant_two.id
 
         form_data['tenant_id'] = tenant.id
+
+        if next:
+            form_data.update({REDIRECT_FIELD_NAME: next})
+
         response = self.client.get(url, form_data)
 
-        self.assertRedirects(response, settings.LOGIN_REDIRECT_URL)
+        if next:
+            expected_url = 'http://testserver%s' % next
+            self.assertEqual(response['location'], expected_url)
+        else:
+            self.assertRedirects(response, settings.LOGIN_REDIRECT_URL)
         self.assertEqual(self.client.session['token']['token']['tenant']['id'],
                          scoped.tenant['id'])
+
+    def test_switch_with_next(self):
+        self.test_switch(next='/next_url')
